#! /usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(app):
    return Info(app)


def quit(app):
    exit(0)


def scroll_down(app):
    app.screen.scroll(1)


def dashport(stdscr):
    app = Dashport(stdscr, scroll=True)
    app.add_control("q", quit)
    app.add_control("KEY_DOWN", scroll_down)
    rows, cols = app.screen.getmaxyx()
    app.widget("title_bar",
               text=f"Title {app.cursor_x},{app.cursor_y}",
               align="top", color=256)
    for i in range(0, app.rows + 56):
        app.print(content=f"Line: {i} {app.cursor_y}")

    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#! /usr/bin/env python3
import curses
from dashport.characters import BoxDrawing


def custom_border(panel, characters):
    rows, cols = panel.getmaxyx()
    ul_corner = characters[0]  # upper left
    ur_corner = characters[1]  # upper right
    ll_corner = characters[2]  # lower left
    lr_corner = characters[3]  # lower right
    left_vert = characters[4]  # left vertical
    rght_vert = characters[5]  # right vertical
    top_horiz = characters[6]  # top horizontal
    btm_horiz = characters[7]  # bottom horizontal
    panel.addstr(0, 0, ul_corner)
    panel.addstr(0, cols - 1, ur_corner)
    panel.addstr(rows - 1, 0, ll_corner)
    panel.insstr(rows - 1, cols - 1, lr_corner)
    for i in range(1, rows - 1):
        panel.addstr(i, 0, left_vert)
        panel.addstr(i, cols - 1, rght_vert)
    for i in range(1, cols - 1):
        panel.addstr(0, i, top_horiz)
        panel.addstr(rows - 1, i, btm_horiz)
    curses.panel.update_panels()


def style(panel, border_style=None, **kwargs):
    boxes = BoxDrawing
    rows, cols = panel.getmaxyx()
    if border_style == 0:
        panel.border(curses.ACS_VLINE,
                     curses.ACS_VLINE,
                     curses.ACS_HLINE,
                     curses.ACS_HLINE,
                     curses.ACS_ULCORNER,
                     curses.ACS_URCORNER,
                     curses.ACS_LLCORNER,
                     curses.ACS_LRCORNER)
    elif border_style == 1:
        panel.border('|', '|', '-', '-', '+', '+', '+', '+')
    elif border_style == 2:
        panel.border('\\', '/', '=', '=', ' ', ' ', ' ', ' ')
    elif border_style == 3:
        characters = [
            chr(boxes.html("double_down_and_right")),
            chr(boxes.html("double_down_and_left")),
            chr(boxes.html("double_up_and_right")),
            chr(boxes.html("double_up_and_left")),
            chr(boxes.html("double_vertical")),
            chr(boxes.html("double_vertical")),
            chr(boxes.html("double_horizontal")),
            chr(boxes.html("double_horizontal"))
        ]
        custom_border(panel, characters)



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(stdscr):
    return Info(stdscr)


def dashport(stdscr):
    app = Dashport(stdscr)

    # This loop refreshes the screen. Don't put any other commands
    # in this loop, instead add to the `app` object above, and let
    # this loop run.
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#! /usr/bin/env python3
import json

with (open('dashport/resources/box_drawing_table.json')) as data:
    box_drawing = json.load(data)
with (open('dashport/resources/block_elements_table.json')) as data:
    block_elements = json.load(data)


class BoxDrawing:
    def data():
        return box_drawing

    def html(character):
        return box_drawing[character][0]

    def unicode(character):
        return box_drawing[character][1]

    def char(character):
        if isinstance(character, str):
            return chr(box_drawing[character][0])
        elif isinstance(character, int):
            if character >= 9472:
                return box_drawing[character][1]
            else:
                return box_drawing[character][0]


class BlockElements:
    def data():
        return block_elements

    def html(character):
        return block_elements[character][0]

    def unicode(character):
        return block_elements[character][1]

    def char(character):
        if isinstance(character, str):
            return chr(block_elements[character][0])
        elif isinstance(character, int):
            if character >= 9472:
                return block_elements[character][1]
            else:
                return block_elements[character][0]



#! /usr/bin/env python3
from dashport.dash import Dashport
from dashport.run import wrap


def quit(app):
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, color_default=8)
    app.add_control("q", quit, case_sensitive=False)
    app.single_panel(border=True)
    app.print("This text is bold.", A_BOLD=True, panel="layout.0", x=1, y=1)
    app.print("This text is reversed.", A_REVERSE=True, panel="layout.0", x=1, y=2)
    app.print("This text is normal.", panel="layout.0", x=1, y=3)
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport)



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(app):
    return Info(app.screen)


def quit(app):
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, color_default=17)
    app.add_control("q", quit, case_sensitive=False)
    app.layout("split_screen_rows", border=True)
    app.print("layout.0", x=1, y=1, panel="layout.0", color=176)
    app.print("layout.1", x=1, y=1, panel="layout.1", color=176)
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#! /usr/bin/env python3
import curses


def single_panel(app, **kwargs):
    border = kwargs.get("border", False)
    border_styles = kwargs.get("border_styles", [0])
    height = kwargs.get("height", app.rows - app.title_offset)
    width = kwargs.get("width", app.cols)
    if isinstance(border, bool):
        border = [border]
    win1, panel1 = app.panel(height=height, length=width, y=0, x=0,
                             scroll=kwargs.get("scroll", False),
                             border=border[0],
                             border_style=border_styles[0])
    curses.panel.update_panels()
    app.screen.refresh()
    app.panel_coords = [[0, 0]]
    app.panel_dimensions = [win1.getmaxyx()]
    return [win1, panel1]


def split_screen_columns(app, **kwargs):
    if not kwargs.get("center_divide_x"):
        split_cols = int(app.cols / 2)
    else:
        split_cols = int(kwargs.get("center_divide_x"))
    border = kwargs.get("border", False)
    border_styles = kwargs.get("border_styles", [0, 0])
    if isinstance(border, bool):
        border = [border] * 2
    win1, panel1 = app.panel(height=app.rows - app.title_offset,
                             length=split_cols, y=0, x=0,
                             scroll=kwargs.get("scroll", False),
                             border=border[0],
                             border_style=border_styles[0])
    win2, panel2 = app.panel(height=app.rows - app.title_offset,
                             length=app.cols - split_cols,
                             y=0,
                             x=split_cols,
                             scroll=kwargs.get("scroll", False),
                             border=border[1],
                             border_style=border_styles[1])
    curses.panel.update_panels()
    app.screen.refresh()
    app.panel_coords = [[0, 0], [0, 0]]
    app.panel_dimensions = [win1.getmaxyx(), win2.getmaxyx()]
    return [win1, win2, panel1, panel2]


def split_screen_rows(app, **kwargs):
    if not kwargs.get("center_divide_y"):
        split_rows = [int(app.rows / 2) - app.title_offset, int(app.rows / 2)]
    else:
        split_rows = [int(kwargs.get("center_divide_y")),
                      app.rows - int(kwargs.get("center_divide_y"))]
    border = kwargs.get("border", False)
    border_styles = kwargs.get("border_styles", [0, 0])
    if isinstance(border, bool):
        border = [border] * 2
    win1, panel1 = app.panel(height=split_rows[0], length=app.cols, y=0, x=0,
                             scroll=kwargs.get("scroll", False),
                             border=border[0],
                             border_style=border_styles[0])
    win2, panel2 = app.panel(height=split_rows[1], length=app.cols,
                             y=split_rows[0],
                             x=0,
                             scroll=kwargs.get("scroll", False),
                             border=border[1],
                             border_style=border_styles[1])
    curses.panel.update_panels()
    app.screen.refresh()
    app.panel_coords = [[0, 0], [0, 0]]
    app.panel_dimensions = [win1.getmaxyx(), win2.getmaxyx()]
    return [win1, win2, panel1, panel2]


def quadrants(app, **kwargs):
    split_rows = int(app.rows / 2) - app.title_offset
    split_cols = int(app.cols / 2)
    border = kwargs.get("border", False)
    border_styles = kwargs.get("border_styles", [0, 0, 0, 0])
    if isinstance(border, bool):
        border = [border] * 4
    win1, panel1 = app.panel(height=split_rows, length=split_cols,
                             y=0, x=0,
                             scroll=kwargs.get("scroll", False),
                             border=border[0],
                             border_style=border_styles[0])
    win2, panel2 = app.panel(height=split_rows, length=split_cols,
                             y=0, x=split_cols,
                             scroll=kwargs.get("scroll", False),
                             border=border[1],
                             border_style=border_styles[1])
    win3, panel3 = app.panel(height=split_rows, length=split_cols,
                             y=split_rows, x=0,
                             scroll=kwargs.get("scroll", False),
                             border=border[2],
                             border_style=border_styles[2])
    win4, panel4 = app.panel(height=split_rows, length=split_cols,
                             y=split_rows, x=split_cols,
                             scroll=kwargs.get("scroll", False),
                             border=border[3],
                             border_style=border_styles[3])
    curses.panel.update_panels()
    app.screen.refresh()
    app.panel_coords = [[0, 0], [0, 0], [0, 0], [0, 0]]
    app.panel_dimensions = [win1.getmaxyx(), win2.getmaxyx(),
                            win3.getmaxyx(), win4.getmaxyx()]
    return [win1, win2, win3, win4, panel1, panel2, panel3, panel4]


def three_panels_vert(app, **kwargs):
    split_rows = int(app.rows / 2) - app.title_offset
    split_cols = int(app.cols / 2)
    long_side = kwargs.get("long_side", "right")
    border = kwargs.get("border", False)
    border_styles = kwargs.get("border_styles", [0, 0, 0])
    if isinstance(border, bool):
        border = [border] * 3
    long_side_width = kwargs.get("long_side_width", None)
    if not long_side_width:
        long_side_width = split_cols
    if long_side == "right":
        win1, panel1 = app.panel(height=split_rows,
                                 length=app.cols - long_side_width,
                                 scroll=kwargs.get("scroll", False),
                                 y=0, x=0, border=border[0],
                                 border_style=border_styles[0])
        win2, panel2 = app.panel(
            height=app.rows - app.title_offset - app.title_bottom_offset,
            length=long_side_width,
            y=0, x=app.cols - long_side_width,
            scroll=kwargs.get("scroll", False),
            border=border[1],
            border_style=border_styles[1])
        win3, panel3 = app.panel(height=split_rows,
                                 length=app.cols - long_side_width,
                                 scroll=kwargs.get("scroll", False),
                                 y=split_rows, x=0, border=border[2],
                                 border_style=border_styles[2])
    elif long_side == "left":
        win1, panel1 = app.panel(
            height=app.rows - app.title_offset - app.title_bottom_offset,
            length=long_side_width,
            scroll=kwargs.get("scroll", False),
            y=0, x=0, border=border[0],
            border_style=border_styles[0])
        win2, panel2 = app.panel(height=split_rows,
                                 length=app.cols - long_side_width,
                                 scroll=kwargs.get("scroll", False),
                                 y=0, x=long_side_width, border=border[1],
                                 border_style=border_styles[1])
        win3, panel3 = app.panel(height=split_rows,
                                 length=app.cols - long_side_width,
                                 y=split_rows, x=long_side_width,
                                 scroll=kwargs.get("scroll", False),
                                 border=border[2],
                                 border_style=border_styles[2])
    curses.panel.update_panels()
    app.screen.refresh()
    app.panel_coords = [[0, 0], [0, 0], [0, 0]]
    app.panel_dimensions = [win1.getmaxyx(), win2.getmaxyx(),
                            win3.getmaxyx()]
    return [win1, win2, win3, panel1, panel2, panel3]


def three_panels_horizontal(app, **kwargs):
    split_rows = int(app.rows / 2) - app.title_offset
    split_cols = int(app.cols / 2)
    long_side = kwargs.get("long_side", "top")
    border = kwargs.get("border", False)
    border_styles = kwargs.get("border_styles", [0, 0, 0])
    if isinstance(border, bool):
        border = [border] * 3
    long_side_height = kwargs.get("long_side_height", None)
    if not long_side_height:
        long_side_height = split_rows
    if long_side == "top":
        win1, panel1 = app.panel(height=long_side_height,
                                 length=app.cols,
                                 scroll=kwargs.get("scroll", False),
                                 y=0, x=0, border=border[0],
                                 border_style=border_styles[0])
        win2, panel2 = app.panel(height=app.rows - long_side_height,
                                 length=split_cols,
                                 scroll=kwargs.get("scroll", False),
                                 y=long_side_height, x=0, border=border[1],
                                 border_style=border_styles[1])
        win3, panel3 = app.panel(height=app.rows - long_side_height,
                                 length=split_cols,
                                 scroll=kwargs.get("scroll", False),
                                 y=long_side_height, x=split_cols,
                                 border=border[2],
                                 border_style=border_styles[2])
    elif long_side == "bottom":
        win1, panel1 = app.panel(height=app.rows - long_side_height,
                                 length=split_cols,
                                 scroll=kwargs.get("scroll", False),
                                 y=0, x=0, border=border[0],
                                 border_style=border_styles[0])
        win2, panel2 = app.panel(height=app.rows - long_side_height,
                                 length=split_cols,
                                 scroll=kwargs.get("scroll", False),
                                 y=0, x=split_cols, border=border[1],
                                 border_style=border_styles[1])
        win3, panel3 = app.panel(height=long_side_height, length=app.cols,
                                 y=app.rows - long_side_height, x=0,
                                 scroll=kwargs.get("scroll", False),
                                 border=border[2],
                                 border_style=border_styles[2])
    curses.panel.update_panels()
    app.screen.refresh()
    app.panel_coords = [[0, 0], [0, 0], [0, 0]]
    app.panel_dimensions = [win1.getmaxyx(), win2.getmaxyx(),
                            win3.getmaxyx()]
    return [win1, win2, win3, panel1, panel2, panel3]



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(app):
    return Info(app.screen)


def quit(app):
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, color_default=176)
    app.add_control("q", quit, case_sensitive=False)
    app.layout("three_panels_horizontal",
               border=[True, True, True], long_side="bottom",
               long_side_height=10)
    app.print("panel layout.0", panel="layout.0")
    app.print(f"Rows: {app.panel_dimensions[0][0]}", x=5, y=2,
              panel="layout.0")
    app.print(f"Columns: {app.panel_dimensions[0][1]}", x=5, y=3,
              panel="layout.0")
    app.print("panel layout.1", panel="layout.1")
    app.print(f"Rows: {app.panel_dimensions[1][0]}", x=5, y=2,
              panel="layout.1")
    app.print(f"Columns: {app.panel_dimensions[1][1]}", x=5, y=3,
              panel="layout.1")
    app.print("panel layout.2", x=1, y=1, panel="layout.2")
    app.print(f"Rows: {app.panel_dimensions[2][0]}", x=5, y=2,
              panel="layout.2")
    app.print(f"Columns: {app.panel_dimensions[2][1]}", x=5, y=3,
              panel="layout.2")
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(app):
    return Info(app.screen)


def quit(app):
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, color_default=176)
    app.add_control("q", quit, case_sensitive=False)
    app.layout("quadrants",
               border=[True, False, True, True],
               border_styles=[0, 1, 1, 3])
    # app.panels["layout"][3].border('M', 'M', '=', '=', ' ', ' ', ' ', ' ')
    app.print("panel: layout.0", x=1, y=1, panel="layout.0",
              A_BOLD=True, A_ITALIC=True)
    app.print(f"Rows: {app.panel_dimensions[0][0]}", x=5, y=2,
              panel="layout.0", A_REVERSE=True)
    app.print(f"Columns: {app.panel_dimensions[0][1]}", x=5, y=3,
              panel="layout.0", A_REVERSE=True)
    app.print("panel: layout.1", x=1, y=1, panel="layout.1",
              A_BOLD=True, A_ITALIC=True)
    app.print(f"Rows: {app.panel_dimensions[1][0]}", x=5, y=2,
              panel="layout.1", A_REVERSE=True)
    app.print(f"Columns: {app.panel_dimensions[1][1]}", x=5, y=3,
              panel="layout.1", A_REVERSE=True)
    app.print("panel: layout.2", x=1, y=1, panel="layout.2",
              A_BOLD=True, A_ITALIC=True)
    app.print(f"Rows: {app.panel_dimensions[2][0]}", x=5, y=2,
              panel="layout.2", A_REVERSE=True)
    app.print(f"Columns: {app.panel_dimensions[2][1]}", x=5, y=3,
              panel="layout.2", A_UNDERLINE=True)
    app.print("panel: layout.3", x=1, y=1, panel="layout.3",
              A_BOLD=True, A_ITALIC=True)
    app.print(f"Rows: {app.panel_dimensions[3][0]}", x=5, y=2,
              panel="layout.3", A_REVERSE=True)
    app.print(f"Columns: {app.panel_dimensions[3][1]}", x=5, y=3,
              panel="layout.3", A_UNDERLINE=True)
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#!/usr/bin/env python3

import curses


def color_names(color_backgrounds=None):
    color_constants = {"default": -1, "black": 0, "maroon": 1, "green": 2,
                       "olive": 3, "navy": 4, "purple": 5, "teal": 6,
                       "silver": 7, "grey": 8, "red": 9, "lime": 10,
                       "yellow": 11, "blue": 12, "fuchsia": 13, "aqua": 14,
                       "white": 15}
    if isinstance(color_backgrounds, list):
        try:
            color_backgrounds = {x: color_constants[x] for x in color_backgrounds}
        except KeyError:
            print(f"One or more colors listed not supported: {color_backgrounds}")
    else:
        color_backgrounds = {"default": -1, "white": 15,
                             "black": 0, "silver": 7, "grey": 8,
                             "red": 9, "lime": 10, "yellow": 11,
                             "blue": 12, "fuchsia": 13, "aqua": 14,
                             "maroon": 1, "green": 2, "navy": 4, "purple": 5}
    color_index = 1
    color_dict = dict()
    for name in list(color_backgrounds.keys()):
        for k in range(-1, 16):
            color_dict = color_foregrounds(color_index, color_backgrounds[name], color_dict, name)
        color_index += 17
    return color_dict, color_backgrounds


def color_foregrounds(color_index, color_int_background, color_dict, color_str_background):
    color_str = ["default", "black", "maroon", "green", "olive", "navy", "purple",
                 "teal", "silver", "grey", "red", "lime", "yellow", "blue", "fuchsia",
                 "aqua", "white"]
    for j in range(-1, 16):
        curses.init_pair(color_index, j, color_int_background)
        color_key = "{}_on_{}".format(color_str[j + 1], color_str_background)
        color_dict[color_key] = color_index
        color_index += 1
    return color_dict


def color_pair_integer(app, color_int=None):
    if not color_int and not app.color_default:
        return curses.color_pair(0)
    elif not color_int:
        return curses.color_pair(app.color_default)
    elif isinstance(color_int, str):
        return curses.color_pair(app.color_names[color_int])
    else:
        return curses.color_pair(color_int)


def format_text(arguments):
    attributes = []
    for arg in list(arguments.keys()):
        if arg.startswith("A_"):
            try:
                attributes.append(getattr(curses, arg))
            except AttributeError:
                pass
    if len(attributes) < 18:
        attributes = attributes + ([0] * (18 - len(attributes)))
    return attributes



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(stdscr):
    return Info(stdscr)


def quit(*args):
    exit(0)


def move_up(app):
    app.screen.clear()
    if app.cursor_y != 0:
        app.cursor_y -= 1


def move_down(app):
    app.screen.clear()
    if app.rows - 1 > app.cursor_y:
        app.cursor_y += 1


def move_left(app):
    app.screen.clear()
    if app.cursor_x != 0:
        app.cursor_x -= 1


def move_right(app):
    app.screen.clear()
    if app.cols - 1 > app.cursor_x:
        app.cursor_x += 1


def dashport(stdscr):
    app = Dashport(stdscr, cursor=False)
    app.add_control("q", quit)
    app.add_control("KEY_UP", move_up)
    app.add_control("KEY_DOWN", move_down)
    app.add_control("KEY_LEFT", move_left)
    app.add_control("KEY_RIGHT", move_right)
    while True:
        app.insstr("X")
        app.addstr("Position: {} {}".format(app.cursor_x, app.cursor_y),
                   x=0, y=app.rows - 1, color=4)
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(stdscr):
    return Info(stdscr)


def quit(stdscr):
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, color_default="yellow_on_blue")
    app.add_control("q", quit, case_sensitive=False)
    # This text won't show because the background will overwrite it.
    app.print("This text won't show.", x=5, y=5)
    # Color 109 is a blue background
    app.background(color="white_on_maroon")
    # This text will show because it comes after the background.
    app.print("printing on top of text")
    app.rectangle(10, 10, 12, 12, color="blue_on_yellow")
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap
from dashport.characters import BoxDrawing

"""
This demo needs at least 110 columns and 40 lines to work.
"""


def set_range(app, custom_range):
    app.cursor_y = 0
    color_names = list(app.color_names.keys())
    app.widget("title_bar", text="Color Palette Test Application".center(app.cols, " "), color="white_on_red")
    app.layout("split_screen_columns", border=[False, True], center_divide_x=21)
    app.print("FORMATS".center(20, " "), x=0, y=18, panel="layout.0", color="black_on_white")
    app.print("press F to change", x=0, y=19, panel="layout.0")
    format_list_y = 20
    for format_style in ["Appearance", "Highlights", "Others"]:
        if app.format_cycle == format_style.lower():
            app.print(f"> {format_style}", x=0, y=format_list_y, panel="layout.0", color="black_on_white")
        else:
            app.print(f"  {format_style}", x=0, y=format_list_y, panel="layout.0")
        format_list_y += 1
    app.panels["layout"][1].clear()
    app.print("BACKGROUND".center(20, " "), x=0, y=0, panel="layout.0", color="black_on_white")
    app.print(BoxDrawing.char("double_horizontal") * 20, x=0, y=1, panel="layout.0")
    panel_list_y = 2
    for background in app.color_backgrounds:
        if background == app.selected_background:
            app.print(f">{background}".ljust(20), x=0, y=panel_list_y, panel="layout.0", A_REVERSE=True)
        else:
            app.print(f"{background}", x=1, y=panel_list_y, panel="layout.0")
        panel_list_y += 1
    if app.format_cycle == "appearance":
        app.print("A_NORMAL", x=1, y=1, panel="layout.1")
        app.print("A_REVERSE", x=22, y=1, panel="layout.1")
        app.print("A_BOLD", x=44, y=1, panel="layout.1")
        app.print("A_ITALIC", x=66, y=1, panel="layout.1")
        app.print("A_DIM", x=1, y=20, panel="layout.1")
        app.print("A_UNDERLINE", x=22, y=20, panel="layout.1")
        app.print("A_BLINK", x=44, y=20, panel="layout.1")
        app.print("A_INVIS", x=66, y=20, panel="layout.1")
        for j in range(custom_range[0], custom_range[1]):
            app.print(f"{color_names[j]}", x=1, y=app.cursor_y + 2, color=color_names[j], panel="layout.1")
            app.print(f"{color_names[j]}", x=22, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_REVERSE=True)
            app.print(f"{color_names[j]}", x=44, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_BOLD=True)
            app.print(f"{color_names[j]}", x=66, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_ITALIC=True)
            app.print(f"{color_names[j]}", x=1, y=app.cursor_y + 21, color=color_names[j], panel="layout.1",
                      A_DIM=True)
            app.print(f"{color_names[j]}", x=22, y=app.cursor_y + 21, color=color_names[j], panel="layout.1",
                      A_UNDERLINE=True)
            app.print(f"{color_names[j]}", x=44, y=app.cursor_y + 21, color=color_names[j], panel="layout.1",
                      A_BLINK=True)
            app.print(f"{color_names[j]}", x=66, y=app.cursor_y + 21, color=color_names[j], panel="layout.1",
                      A_INVIS=True)
            app.cursor_y += 1

    if app.format_cycle == "highlights":
        app.print("A_VERTICAL", x=1, y=1, panel="layout.1")
        app.print("A_HORIZONTAL", x=22, y=1, panel="layout.1")
        app.print("A_LEFT", x=44, y=1, panel="layout.1")
        app.print("A_LOW", x=1, y=20, panel="layout.1")
        app.print("A_RIGHT", x=22, y=20, panel="layout.1")
        app.print("A_TOP", x=44, y=20, panel="layout.1")
        for j in range(custom_range[0], custom_range[1]):
            app.print(f"{color_names[j]}", x=1, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_VERTICAL=True)
            app.print(f"{color_names[j]}", x=22, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_HORIZONTAL=True)
            app.print(f"{color_names[j]}", x=44, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_LEFT=True)
            app.print(f"{color_names[j]}", x=1, y=app.cursor_y + 21, color=color_names[j], panel="layout.1",
                      A_LOW=True)
            app.print(f"{color_names[j]}", x=22, y=app.cursor_y + 21, color=color_names[j], panel="layout.1",
                      A_RIGHT=True)
            app.print(f"{color_names[j]}", x=44, y=app.cursor_y + 21, color=color_names[j], panel="layout.1",
                      A_TOP=True)
            app.cursor_y += 1

    if app.format_cycle == "others":
        app.print("A_ALTCHARSET", x=1, y=1, panel="layout.1")
        app.print("A_PROTECT", x=22, y=1, panel="layout.1")
        app.print("A_CHARTEXT", x=44, y=1, panel="layout.1")
        for j in range(custom_range[0], custom_range[1]):
            app.print(f"{color_names[j]}", x=1, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_ALTCHARSET=True)
            app.print(f"{color_names[j]}", x=22, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_PROTECT=True)
            app.print(f"{color_names[j]}", x=44, y=app.cursor_y + 2, color=color_names[j], panel="layout.1",
                      A_CHARTEXT=True)
            app.cursor_y += 1


def default_range(app):
    set_range(app, [1, 17])


def next_background(app):
    backgrounds = list(app.color_backgrounds.keys())
    if app.selected_background != "purple":
        current_index = backgrounds.index(app.selected_background)
        app.selected_background = backgrounds[current_index+1]


def prev_background(app):
    backgrounds = list(app.color_backgrounds.keys())
    if app.selected_background != "default":
        current_index = backgrounds.index(app.selected_background)
        app.selected_background = backgrounds[current_index - 1]


def cycle_format(app):
    cycle_formats = ["appearance", "highlights", "others"]
    current_index = cycle_formats.index(app.format_cycle)
    if current_index < 2:
        app.format_cycle = cycle_formats[current_index + 1]
    else:
        app.format_cycle = cycle_formats[0]



def white_range(app):
    set_range(app, [17, 34])


def black_range(app):
    set_range(app, [34, 51])


def silver_range(app):
    set_range(app, [51, 68])


def grey_range(app):
    set_range(app, [68, 85])


def red_range(app):
    set_range(app, [85, 102])


def lime_range(app):
    set_range(app, [102, 119])


def yellow_range(app):
    set_range(app, [119, 136])


def blue_range(app):
    set_range(app, [136, 153])


def fuchsia_range(app):
    set_range(app, [153, 170])


def aqua_range(app):
    set_range(app, [170, 187])


def maroon_range(app):
    set_range(app, [187, 204])


def green_range(app):
    set_range(app, [204, 221])


def navy_range(app):
    set_range(app, [221, 238])


def purple_range(app):
    set_range(app, [238, 255])


def exit_program(*args):
    exit(0)


def info(stdscr):
    return Info(stdscr)


def dashport(stdscr):
    app = Dashport(stdscr, color_default=9)
    app.format_cycle = "appearance"
    app.add_control("q", exit_program, case_sensitive=False)
    app.add_control("F", cycle_format, case_sensitive=False)
    app.set_color_range = [1, 17]
    app.selected_background = "default"
    app.add_control("KEY_DOWN", next_background)
    app.add_control("KEY_UP", prev_background)
    while True:
        globals()["{}_range".format(app.selected_background)](app)
        app.refresh()



if __name__ == '__main__':
    wrap(dashport, info)



#!/usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap


def info(app):
    return Info(app.screen)


def quit(app):
    exit(0)


def four_way_split(app):
    app.layout("quadrants", border=True, height=app.rows - 2)
    app.print("panel 0", panel="layout.0")
    app.print(f"Rows: {app.panel_dimensions[0][0]}", x=5, y=2,
              panel="layout.0")
    app.print(f"Columns: {app.panel_dimensions[0][1]}", x=5, y=3,
              panel="layout.0")
    app.print("panel 1", panel="layout.1")
    app.print(f"Rows: {app.panel_dimensions[1][0]}", x=5, y=2,
              panel="layout.1")
    app.print(f"Columns: {app.panel_dimensions[1][1]}", x=5, y=3,
              panel="layout.1")
    app.print("panel 2", panel="layout.2")
    app.print(f"Rows: {app.panel_dimensions[2][0]}", x=5, y=2,
              panel="layout.2")
    app.print(f"Columns: {app.panel_dimensions[2][1]}", x=5, y=3,
              panel="layout.2")
    app.print("panel 3", panel="layout.3")
    app.print(f"Rows: {app.panel_dimensions[3][0]}", x=5, y=2,
              panel="layout.3")
    app.print(f"Columns: {app.panel_dimensions[3][1]}", x=5, y=3,
              panel="layout.3")


def three_way_split(app):
    app.layout("three_panels_vert", border=True, long_side="right",
               long_side_width=100)
    app.print("panel 0", panel="layout.0")
    app.print(f"Rows: {app.panel_dimensions[0][0]}", x=5, y=2,
              panel="layout.0")
    app.print(f"Columns: {app.panel_dimensions[0][1]}", x=5, y=3,
              panel="layout.0")
    app.print("panel 1", panel="layout.1")
    app.print(f"Rows: {app.panel_dimensions[1][0]}", x=5, y=2,
              panel="layout.1")
    app.print(f"Columns: {app.panel_dimensions[1][1]}", x=5, y=3,
              panel="layout.1")
    app.print("panel 2", panel="layout.2")
    app.print(f"Rows: {app.panel_dimensions[2][0]}", x=5, y=2,
              panel="layout.2")
    app.print(f"Columns: {app.panel_dimensions[2][1]}", x=5, y=3,
              panel="layout.2")


def dashport(stdscr):
    app = Dashport(stdscr, color_default=176,  scroll=True)
    app.add_control("q", quit, case_sensitive=False)
    app.widget("title_bar",
               text="Press 4 to split 4 ways, 3 to split 3 ways, "
               "Q to quit.",
               align="top", color=256)
    three_way_split(app)
    app.add_control("4", four_way_split)
    app.add_control("3", three_way_split)
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#! /usr/bin/env python3
from curses.textpad import Textbox
import curses


def user_prompt(app, **kwargs):
    command_line_entered = []
    cursor_x = kwargs.get("x", 0)
    cursor_y = kwargs.get("y", 0)

    def validate_text(x):
        invalid_chars = [127, 260, 261, 10]
        if x == 260 and app.user_prompt_position != 0:
            app.user_prompt_position -= 1
        if x == 261:
            app.user_prompt_position += 1
        if x not in invalid_chars and curses.ascii.isprint(x):
            command_line_entered.insert(app.user_prompt_position, chr(x))
            app.user_prompt_position += 1
        if x == 127 and len(command_line_entered) > 0:
            try:
                command_line_entered.pop(app.user_prompt_position - 1)
            except IndexError:
                pass
            app.user_prompt_position -= 1
        app.screen.refresh()
        if x == 127:
            x = 263
        if x == 262:
            x = 1
        if x == 260:
            x = 2
        if x == 261:
            x = 6
        if x == 10:
            x = 7
        if x == 1:
            app.user_prompt_position = 0
        app.current_command = "".join(command_line_entered)
        return x

    if not app.panels.get("prompt"):
        win1, panel1 = app.panel(height=kwargs.get("height", 1),
                                 length=kwargs.get("length", 20),
                                 y=cursor_y, x=cursor_x)
        app.panels["prompt"] = [win1, panel1]
        app.panel_coords.append([0, 0])
    app.screen.move(cursor_y, cursor_x)
    curses.panel.update_panels()
    app.screen.refresh()
    app.screen.move(cursor_y, cursor_x)
    tb = Textbox(app.panels["prompt"][0], insert_mode=True)
    curses.curs_set(True)
    tb.edit(validate_text)
    curses.curs_set(False)
    app.panels["prompt"].clear()
    return app.current_command



#! /usr/bin/env python3
from dashport.dash import Dashport
from dashport.run import wrap
from dashport.prompt import user_prompt


def quit():
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, color_default=8)
    app.layout("single_panel", border=False, scroll=True, height=app.rows - 1)
    app.commands = []
    while True:
        while True:
            app.user_prompt_position = 1
            app.print(content="{}".format(app.current_command),
                      x=0, y=app.rows - 2, panel="layout.0")
            app.addstr(">", x=0, y=app.rows - 1)
            command = user_prompt(app, x=2, y=app.rows - 1)
            if command == "quit":
                quit()
            else:
                app.panels["layout"][0].scroll(1)
        app.refresh()


if __name__ == '__main__':
    wrap(dashport)



#! /usr/bin/env python3
import curses
from curses import wrapper


def error_message(e, screen):
    print("Curses Error: {}".format(e))
    # Print debug info to stdout
    info = wrapper(screen)
    print("Curses crashed. It may be due to your terminal window "
          "being too small or referencing a position "
          "that does not exist. Here's the debug info:")
    print("# of rows: {}".format(info.rows))
    print("# of cols: {}".format(info.cols))


def wrap(dashport, info=None):
    try:
        wrapper(dashport)
    except curses.error as e:
        if info:
            error_message(e, info)
        else:
            print(e)
            raise



#! /usr/bin/env python3
from dashport.dash import Dashport, Info
from dashport.run import wrap
import curses


def info(stdscr):
    return Info(stdscr)


def quit(*args):
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, scroll=True)
    app.add_control("q", quit)
    app.print("Press 'q' to exit.")
    app.print("")
    curses_constants = [{"name": x, "integer": getattr(curses, x)}
                        for x in dir(curses) if x.startswith("ACS_")]
    for i in range(0, 21):
        app.print("{}".format(curses_constants[i]["name"]))
        app.addstr("{} ".format(hex(curses_constants[i]["integer"])),
                   x=15, y=app.cursor_y - 1)
        app.screen.addch(getattr(curses, curses_constants[i]["name"]))
    count_line = 1
    for i in range(21, 43):
        app.print("{}".format(curses_constants[i]["name"]), x=30, y=count_line)
        app.addstr("{} ".format(hex(curses_constants[i]["integer"])),
                   x=44, y=count_line)
        app.screen.addch(getattr(curses, curses_constants[i]["name"]))
        count_line += 1
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport, info)



#! /usr/bin/env python3

from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='dashport',
    setup_requires=['setuptools_scm'],
    use_scm_version=True,
    description='Dashport curses wrapper for Python',
    long_description_content_type="text/markdown",
    long_description=long_description,
    author='JP Etcheber',
    author_email='jetcheber@gmail.com',
    url='https://github.com/numbertheory/dashport',
    license_files=('LICENSE'),
    packages=find_packages(exclude=('tests', 'docs', 'examples'))
)



#!/usr/bin/env python3
from dashport.dash import Dashport
from dashport.run import wrap

def dashport(stdscr):
    app = Dashport(stdscr, color_default="green_on_white")
    app.print("hello world", color="red_on_white")
    app.print("plain text, colored with color_default")
    app.print("black text on blue background", color="black_on_blue")
    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport)



#!/usr/bin/env python3
from dashport.dash import Dashport
from dashport.run import wrap
from dashport.characters import BoxDrawing, BlockElements


class Select:
    def __init__(self):
        self.row = 0
        self.column = 0
        self.current_view = "block_elements"
        self.cv_index = 0
        self.last_cell = {"line_drawing": [12, 7],
                          "block_elements": [3, 1]}
        self.last_cell_values = {"line_drawing": [11, 12, 9],
                                 "block_elements": [2, 3, 9]}
        first_key = sorted(
            BoxDrawing.data().items(),
            key=self.by_value)[0][0]
        self.current_selection = first_key

    def by_value(self, item):
        return item[1]


selector = Select()


def quit(app):
    exit(0)


def by_value(item):
    return item[1]


def move_up(app):
    if 0 < selector.row:
        selector.row -= 1


def move_down(app):
    current_pos = [selector.column, selector.row]
    last_cell = selector.last_cell[selector.current_view]
    last_cell_values = selector.last_cell_values[selector.current_view]
    if selector.row < last_cell_values[2] and current_pos != last_cell:
        selector.row += 1


def move_left(app):
    if 0 < selector.column:
        selector.column -= 1


def move_right(app):
    projected_pos = [selector.column + 1, selector.row]
    last_cell = selector.last_cell[selector.current_view]
    last_cell_values = selector.last_cell_values[selector.current_view]
    check_col = projected_pos[0] <= last_cell[0]
    if projected_pos[0] > last_cell_values[0]:
        check_row = projected_pos[1] <= last_cell[1]
    else:
        check_row = True
    if selector.column < last_cell_values[1] and check_col and check_row:
        selector.column += 1


def change_view(app):
    app.screen.clear()
    app.widget("title_bar", text="Q = quit, B = Next page",
               align="bottom", color=121)
    if selector.current_view == "line_drawing":
        selector.current_view = "block_elements"
        selector.cv_index = 0
        selector.column = 0
        selector.row = 0
    else:
        selector.current_view = "line_drawing"
        selector.cv_index = 1
        selector.column = 0
        selector.row = 0


def table_drawing(app, selection, table):
    index_of_everything = [0, 0]
    column = 3
    row = 3
    table_elements = table["elements"].data()

    for key, value in sorted(table_elements.items(), key=by_value):
        if index_of_everything == selection:
            selector.current_selection = key
            app.print(panel="layout.0",
                      x=column - 1,
                      y=row,
                      color=121,
                      content=" {} ".format(chr(value[0])))
        else:
            app.print(panel="layout.0",
                      x=column - 1,
                      y=row,
                      content=" {} ".format(chr(value[0])))

        if row >= 20:
            row = 3
            column += 4
            index_of_everything[0] += 1
            index_of_everything[1] = 0
        else:
            row += 2
            index_of_everything[1] += 1
    python_content = "Name: {}".format(selector.current_selection)
    extra_space = " " * (app.cols - len(python_content))
    app.print(panel="layout.0",
              content=python_content + extra_space, x=2, y=24)
    app.print(panel="layout.0",
              content="HTML: &#{};".format(
                table["elements"].html(selector.current_selection)),
              x=2, y=25)
    app.print(panel="layout.0",
              content="Unicode: U+{}".format(
                table["elements"].unicode(selector.current_selection)),
              x=2, y=26)


def dashport(stdscr):
    app = Dashport(stdscr)
    height = app.rows - 2
    app.widget("title_bar", text="Q = quit, B = Next page",
               align="bottom", color=121)
    app.layout("single_panel", scroll=True, height=height)
    app.add_control("q", quit)
    app.add_control("KEY_UP", move_up)
    app.add_control("KEY_DOWN", move_down)
    app.add_control("KEY_LEFT", move_left)
    app.add_control("KEY_RIGHT", move_right)
    app.add_control("B", change_view, case_sensitive=False)
    table = [{"elements": BlockElements, "name": "block_elements",
              "display_name": "Block Elements"},
             {"elements": BoxDrawing, "name": "line_drawing",
              "display_name": "Box Drawing"}]
    while True:
        table_drawing(app, [selector.column, selector.row],
                      table[selector.cv_index])
        app.print(panel="layout.0",
                  content="{} Group".format(
                    table[selector.cv_index]["display_name"]),
                  x=1, y=1)
        app.refresh()


if __name__ == '__main__':
    wrap(dashport)



#! /usr/bin/env python3
import curses
import curses.panel
from dashport.colors import color_pair_integer as cpi
from dashport.colors import color_names, format_text
from dashport import layout, widgets, borders


class Info():
    def __init__(self, stdscr, **kwargs):
        self.screen = stdscr
        self.rows, self.cols = self.screen.getmaxyx()
        self.cursor_y = 0
        self.cursor_x = 0

    def print(self, content="", **kwargs):
        set_y = kwargs.get("y", self.cursor_y)
        set_x = kwargs.get("x", self.cursor_x)
        ending = kwargs.get("end", "\n")
        self.screen.addstr(set_y, set_x, content)
        if ending == "\n":
            self.cursor_y += 1


class Dashport():
    def __init__(self, stdscr, **kwargs):
        self.screen = stdscr
        self.title_bottom_offset = 0
        self.curs_set(kwargs.get("cursor", 0))
        self.rows, self.cols = self.screen.getmaxyx()
        self.window = curses.newwin(self.rows + 1, self.cols)
        self.controls = dict()
        self.cursor_x = 0
        self.cursor_y = 0
        self.panels = dict()
        self.panel_dimensions = []
        self.panel_scroll = []
        self.panel_border = []
        self.panel_border_styles = []
        self.title_offset = 0
        self.top_title_row = None
        self.bottom_title_row = self.rows - 1
        self.scroll_screen = False
        self.current_command = ""
        if kwargs.get("scroll"):
            self.screen.scrollok(True)
            self.scroll_screen = True
        self.screen.setscrreg(0, self.rows - 1)
        curses.start_color()
        curses.use_default_colors()
        if kwargs.get("color_map", True):
            self.color_names, self.color_backgrounds = color_names(kwargs.get("color_names"))
        else:
            self.color_names = dict()
            self.color_backgrounds = []
        self.color_default = kwargs.get("color_default", None)


    def curs_set(self, set_cursor):
        curses.curs_set(set_cursor)

    def add_control(self, control_key, func, case_sensitive=True):
        """
        Binds a keyboard key to a function in the program.
        :param control_key: The string of the key that will be
                           bound to the function.
        :param func: The function that will be executed when
                    this key is pressed
        :param case_sensitive: If set to ``False`` this will add both
                               upper and lowercase characters of the
                               control_key to the same function
        """
        if case_sensitive:
            self.controls[control_key] = func
        if not case_sensitive:
            self.controls[control_key.upper()] = func
            self.controls[control_key.lower()] = func

    def control_keys(self):
        key_pressed = self.screen.getkey()
        if key_pressed and self.controls:
            if key_pressed in self.controls:
                self.controls[key_pressed](self)

    def panel(self, **kwargs):
        height = kwargs.get("height")
        length = kwargs.get("length")
        y = kwargs.get("y")
        x = kwargs.get("x")
        border = kwargs.get("border")
        border_style = kwargs.get("border_style", 0)
        enable_scroll = kwargs.get("scroll", False)
        win = curses.newwin(height, length, y + self.title_offset, x)
        if enable_scroll:
            win.scrollok(True)
            if border:
                win.setscrreg(1, height - 2)
            else:
                win.setscrreg(0, height - 1)
        if border:
            borders.style(win, border_style)
        panel = curses.panel.new_panel(win)
        self.panel_border.append(border)
        self.panel_border_styles.append(border_style)
        curses.panel.update_panels()
        return win, panel

    def refresh(self):
        curses.panel.update_panels()
        self.control_keys()
        self.screen.refresh()

    def print(self, content="",  color=None, **kwargs):
        """
        Print behaves like a TTY print function. At the end of the content
        string the cursor goes to the next line by default.
        """
        set_y = kwargs.get("y", self.cursor_y) + self.title_offset
        set_x = kwargs.get("x", self.cursor_x)
        if kwargs.get("panel"):
            panel = [x for x in kwargs.get("panel").split(".")
                     if not x.isdigit()]
            panel.append(int(kwargs.get("panel").split(".")[1]))
        else:
            panel = None
        ending = kwargs.get("end", "\n")
        format_text_list = format_text(kwargs)
        if set_y == self.top_title_row:
            set_y += 1
        if set_y == self.bottom_title_row and self.scroll_screen:
            self.screen.scroll(1)
            ending = ""
        if not color:
            color = self.color_default
        elif isinstance(color, str):
            color = self.color_names[color]
        if not isinstance(panel, list):
            self.screen.addstr(set_y, set_x, content,
                               cpi(self, color)
                               | format_text_list[0]
                               | format_text_list[1]
                               | format_text_list[2]
                               | format_text_list[3]
                               | format_text_list[4]
                               | format_text_list[5]
                               | format_text_list[6]
                               | format_text_list[7]
                               | format_text_list[8]
                               | format_text_list[9]
                               | format_text_list[10]
                               | format_text_list[11]
                               | format_text_list[12]
                               | format_text_list[13]
                               | format_text_list[14]
                               | format_text_list[15]
                               | format_text_list[16]
                               | format_text_list[17])
            if ending == "\n" and (
                set_y == self.cursor_y + self.title_offset and
                    set_x == self.cursor_x):
                self.cursor_y += 1
        else:
            panel_y = kwargs.get("y", self.panel_coords[panel[1]][1])
            panel_x = kwargs.get("x", self.panel_coords[panel[1]][0])
            if self.panel_border[panel[1]]:
                border_offset = 2
            else:
                border_offset = 1
            if panel_y > self.panel_dimensions[panel[1]][0] - border_offset:
                self.panels[panel[0]][panel[1]].scroll(1)
                ending = ""
                panel_y = self.panel_dimensions[panel[1]][0] - border_offset
            self.panels[panel[0]][panel[1]].addstr(
                panel_y, panel_x, content,
                cpi(self, color)
                | format_text_list[0]
                | format_text_list[1]
                | format_text_list[2]
                | format_text_list[3]
                | format_text_list[4]
                | format_text_list[5]
                | format_text_list[6]
                | format_text_list[7]
                | format_text_list[8]
                | format_text_list[9]
                | format_text_list[10]
                | format_text_list[11]
                | format_text_list[12]
                | format_text_list[13]
                | format_text_list[14]
                | format_text_list[15]
                | format_text_list[16]
                | format_text_list[17])
            if self.panel_border[panel[1]]:
                borders.style(self.panels[panel[0]][panel[1]],
                              self.panel_border_styles[panel[1]])
            curses.panel.update_panels()
            self.screen.refresh()

            if ending == "\n" and (
                panel_y == self.panel_coords[panel[1]][1] and
                    panel_x == self.panel_coords[panel[1]][0]):
                self.panel_coords[panel[1]][1] += 1

    def insstr(self, char="", x=None, y=None, color=None, **kwargs):
        """
        Add a character to the cursor position of the screen [x, y].
        Best used when you don't want the cursor to advance right when
        a character is placed.

        If no cursor position is specified, the current cursor position
        is used.
        """
        format_text_list = format_text(kwargs)
        if not x:
            x = self.cursor_x
        if not y:
            y = self.cursor_y + self.title_offset
        if not color:
            color = self.color_default
        elif isinstance(color, str):
            color = self.color_names[color]
        self.screen.insstr(y, x, char, cpi(self, color)
                           | format_text_list[0]
                           | format_text_list[1]
                           | format_text_list[2]
                           | format_text_list[3]
                           | format_text_list[4]
                           | format_text_list[5]
                           | format_text_list[6]
                           | format_text_list[7]
                           | format_text_list[8]
                           | format_text_list[9]
                           | format_text_list[10]
                           | format_text_list[11]
                           | format_text_list[12]
                           | format_text_list[13]
                           | format_text_list[14]
                           | format_text_list[15]
                           | format_text_list[16]
                           | format_text_list[17])

    def addstr(self, content, x, y, color=None, **kwargs):
        """
        Adds a string to the location specified by x, y coordinates. Similar
        to this class's print method, but with no shifting of content already
        on the screen, and the cursor position does not move.
        """
        format_text_list = format_text(kwargs)
        if not color:
            color = self.color_default
        elif isinstance(color, str):
            color = self.color_names[color]
        self.screen.addstr(y + self.title_offset, x, content,
                           cpi(self, color)
                           | format_text_list[0]
                           | format_text_list[1]
                           | format_text_list[2]
                           | format_text_list[3]
                           | format_text_list[4]
                           | format_text_list[5]
                           | format_text_list[6]
                           | format_text_list[7]
                           | format_text_list[8]
                           | format_text_list[9]
                           | format_text_list[10]
                           | format_text_list[11]
                           | format_text_list[12]
                           | format_text_list[13]
                           | format_text_list[14]
                           | format_text_list[15]
                           | format_text_list[16]
                           | format_text_list[17])

    def rectangle(self, x, y, width, height, color=None):
        """
        Draw a filled in rectangle on the screen.
        """
        for j in range(y + self.title_offset,
                       y + height - self.title_bottom_offset):
            for i in range(x, x + width):
                self.screen.addstr(j, i, " ", cpi(self, color))

    def background(self, color):
        """
        Fill the background with a color
        """
        for j in range(0 + self.title_offset,
                       self.rows - self.title_bottom_offset):
            for i in range(0, self.cols):
                self.screen.insstr(j, i, " ", cpi(self, color))

    def layout(self, layout_name, **kwargs):
        self.panels["layout"] = getattr(layout, layout_name)(self, **kwargs)

    def widget(self, widget_name, **kwargs):
        getattr(widgets, widget_name)(self, **kwargs)



#!/usr/bin/env python3
from dashport.dash import Dashport
from dashport.run import wrap
from dashport.borders import custom_border
from dashport.characters import BoxDrawing as boxes


def quit(app):
    exit(0)


def dashport(stdscr):
    app = Dashport(stdscr, color_default=176)
    app.add_control("q", quit, case_sensitive=False)
    app.layout("single_panel")
    border_characters = [
            chr(boxes.html("double_down_and_right")),  # upper left
            chr(boxes.html("double_down_and_left")),   # upper right
            chr(boxes.html("double_up_and_right")),    # lower left
            chr(boxes.html("double_up_and_left")),     # lower right
            chr(boxes.html("double_vertical")),        # left vertical
            chr(boxes.html("double_vertical")),        # right vertical
            chr(boxes.html("double_horizontal")),      # top horizontal
            chr(boxes.html("double_horizontal"))       # bottom horizontal
    ]
    custom_border(app.panels["layout"][0], border_characters)
    app.print("panel 0", x=5, y=2, panel="layout.0")
    app.print(f"Rows: {app.panel_dimensions[0][0]}",
              x=5, y=3, panel="layout.0")
    app.print(f"Columns: {app.panel_dimensions[0][1]}",
              x=5, y=4, panel="layout.0")

    while True:
        app.refresh()


if __name__ == '__main__':
    wrap(dashport)



#! /usr/bin/env python3

def title_bar(app, **kwargs):
    align = kwargs.get("align", "top")
    text = kwargs.get("text", "")
    color = kwargs.get("color", app.color_default)
    if align == "top":
        app.insstr(text.ljust(app.cols), x=0, y=0, color=color)
        app.title_offset = 1
        app.screen.setscrreg(1, app.rows - 1)
        app.top_title_row = 0
    elif align == "bottom":
        app.insstr(text.ljust(app.cols - 1),
                   x=0, y=app.rows - app.title_offset - 1, color=color)
        app.insstr(" ", x=app.cols - 1, y=app.rows - app.title_offset - 1,
                   color=color)
        app.bottom_title_row = app.rows - app.title_offset - 2
        app.title_bottom_offset = 1
        app.screen.setscrreg(0, app.rows - 2)
